'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var di = require('@stringke/sigi-di');

const MAX_OBJECT_DEPTH = 100;
const CloneTrait = Symbol('clone-trait');
function cloneDeepPoj(target) {
    let depth = 0;
    function _clone(obj) {
        if (depth >= MAX_OBJECT_DEPTH) {
            throw new TypeError('Max clone depth reached');
        }
        depth++;
        if (typeof obj === 'object') {
            if (obj[CloneTrait]) {
                return obj[CloneTrait]();
            }
            if (Array.isArray(obj)) {
                return obj.map((item) => cloneDeepPoj(item));
            }
            if (!isPlainObject(obj)) {
                throw new TypeError('Only plain object supported to sync to Sigi');
            }
            const result = Object.create(null);
            for (const [key, value] of Object.entries(obj)) {
                result[key] = _clone(value);
            }
            return result;
        }
        if (typeof obj === 'function' || typeof obj === 'symbol') {
            throw new TypeError(`${typeof obj} type could not be cloned`);
        }
        return obj;
    }
    return _clone(target);
}
function isPlainObject(obj) {
    return Object.prototype.toString.call(obj) === '[object Object]';
}

const reactive = (EffectModuleConstructor, componentOptions) => {
    const effectModule = di.rootInjector.getInstance(EffectModuleConstructor);
    const store = effectModule.store;
    const initialState = store.state;
    const statePassToVue = { ...initialState };
    const subscription = store.state$.subscribe((state) => {
        Object.assign(statePassToVue, state);
    });
    const actionsCreator = effectModule.getActions();
    const dispatchProps = Object.keys(actionsCreator).reduce((acc, cur) => {
        acc[cur] = (payload) => {
            const action = actionsCreator[cur](payload);
            store.dispatch(action);
        };
        return acc;
    }, Object.create(null));
    const { beforeDestroy: originalBeforeDestroy } = componentOptions;
    componentOptions.beforeDestroy = function beforeDestroy() {
        subscription.unsubscribe();
        typeof originalBeforeDestroy === 'function' ? originalBeforeDestroy.call(this) : void 0;
    };
    componentOptions.methods = Object.assign(dispatchProps, componentOptions.methods ?? {});
    const { data: originalData } = componentOptions;
    componentOptions.data = function data() {
        if (typeof originalData === 'function') {
            return Object.assign(statePassToVue, originalData.call(this));
        }
        return statePassToVue;
    };
    const syncToSigi = componentOptions.syncToSigi ?? [];
    const { beforeUpdate: originalBeforeUpdate } = componentOptions;
    componentOptions.beforeUpdate = function beforeUpdate() {
        let latestState = store.state;
        let changed = false;
        for (const prop of syncToSigi) {
            if (!(prop in statePassToVue) && process.env.NODE_ENV === 'development') {
                console.warn(`${prop} in syncToSigi option is not existed in defaultProps property in Sigi module`);
            }
            const propValue = this[prop];
            if (propValue !== latestState[prop]) {
                latestState = { ...latestState, [prop]: cloneDeepPoj(propValue) };
                changed = true;
            }
        }
        if (changed) {
            store.state$.next(latestState);
        }
        if (typeof originalBeforeUpdate === 'function') {
            originalBeforeUpdate.call(this);
        }
    };
    return componentOptions;
};

exports.reactive = reactive;
//# sourceMappingURL=index.js.map
