import { __awaiter, __decorate, __metadata } from "tslib";
import 'reflect-metadata';
import { EffectModule, Module, ImmerReducer, Effect } from '@stringke/sigi-core';
import { Test, SigiTestModule } from '@stringke/sigi-testing';
import { Observable, timer } from 'rxjs';
import { mergeMap, map, switchMap } from 'rxjs/operators';
import * as Sinon from 'sinon';
import Vue from 'vue';
import { reactive } from '../index';
let VueTestingModule = class VueTestingModule extends EffectModule {
    constructor() {
        super(...arguments);
        this.defaultState = { count: 0, name: '' };
    }
    setCount(state, count) {
        state.count = count;
    }
    setName(state, name) {
        state.name = name;
    }
    getCount(payload$) {
        return payload$.pipe(mergeMap(() => timer(20).pipe(map(() => this.getActions().setCount(1)))));
    }
    asyncSetName(payload$) {
        return payload$.pipe(switchMap((name) => timer(20).pipe(map(() => this.getActions().setName(name)))));
    }
};
__decorate([
    ImmerReducer(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Number]),
    __metadata("design:returntype", void 0)
], VueTestingModule.prototype, "setCount", null);
__decorate([
    ImmerReducer(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", void 0)
], VueTestingModule.prototype, "setName", null);
__decorate([
    Effect(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Observable]),
    __metadata("design:returntype", void 0)
], VueTestingModule.prototype, "getCount", null);
__decorate([
    Effect(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Observable]),
    __metadata("design:returntype", void 0)
], VueTestingModule.prototype, "asyncSetName", null);
VueTestingModule = __decorate([
    Module('VueTestingModule')
], VueTestingModule);
describe('VueJS reactive binding', () => {
    const options = reactive(VueTestingModule, {});
    let vm = new Vue(options);
    let testingStub;
    let timer;
    beforeEach(() => {
        const testingModule = Test.createTestingModule({
            TestModule: SigiTestModule,
        }).compile();
        vm = new Vue(reactive(VueTestingModule, {}));
        testingStub = testingModule.getTestingStub(VueTestingModule);
        timer = Sinon.useFakeTimers();
    });
    afterEach(() => {
        timer.restore();
    });
    it('should create vue component options', () => {
        expect(vm).toBeInstanceOf(Vue);
    });
    it('should be able to call reducer', () => {
        const count = 1;
        vm.setCount(count);
        expect(testingStub.getState().count).toBe(count);
    });
    it('should be able to call effect', () => {
        const name = 'VueJS';
        vm.asyncSetName(name);
        timer.runAll();
        expect(testingStub.getState().name).toBe(name);
    });
    it('state should map to vue data', () => {
        vm.getCount();
        timer.runAll();
        expect(vm.count).toBe(1);
    });
    it('should stop observe state change after destroy', () => __awaiter(void 0, void 0, void 0, function* () {
        vm.$destroy();
        yield vm.$nextTick();
        const count = 1000;
        vm.setCount(count);
        expect(vm.count).not.toBe(count);
    }));
    it('should be able to merge state to data', () => {
        const reactiveOptions = reactive(VueTestingModule, {
            data() {
                return {
                    foo: '1',
                };
            },
            methods: {
                setFoo() {
                    this.foo = '2';
                },
            },
        });
        const vm = new Vue(reactiveOptions);
        expect(vm.foo).toBe('1');
        vm.setFoo();
        expect(vm.foo).toBe('2');
    });
    it('should be able to syncToSigi', () => {
        const newName = 'fake new name';
        const reactiveOptions = reactive(VueTestingModule, {
            data() {
                return {
                    foo: 1,
                };
            },
            syncToSigi: ['name'],
            render(h) {
                return h('div', [h('span', this.name)]);
            },
        });
        const vm = new Vue(reactiveOptions);
        vm.name = newName;
        vm.$options.beforeUpdate[0].call(vm);
        expect(testingStub.getState().name).toBe(newName);
    });
    it('should merge original beforeUpdate lifecycle', () => {
        const spy = Sinon.spy();
        const reactiveOptions = reactive(VueTestingModule, {
            syncToSigi: ['name'],
            beforeUpdate: spy,
        });
        const vm = new Vue(reactiveOptions);
        vm.$options.beforeUpdate[0].call(vm);
        expect(spy.callCount).toBe(1);
    });
    it('should warn if property which syncToSigi not existed', () => {
        const spy = Sinon.spy(console, 'warn');
        const reactiveOptions = reactive(VueTestingModule, {
            syncToSigi: ['name-n'],
        });
        const vm = new Vue(reactiveOptions);
        const { NODE_ENV } = process.env;
        process.env.NODE_ENV = 'development';
        vm.$options.beforeUpdate[0].call(vm);
        expect(spy.callCount).toBe(1);
        spy.restore();
        process.env.NODE_ENV = NODE_ENV;
    });
    it('should ignore non-function data property', () => {
        const reactiveOptions = reactive(VueTestingModule, {
            data: {
                foo: '1',
            },
        });
        const vm = new Vue(reactiveOptions);
        expect(vm['foo']).toBeUndefined();
    });
    it('should call original beforeDestroy', () => __awaiter(void 0, void 0, void 0, function* () {
        const spy = Sinon.spy();
        const reactiveOptions = reactive(VueTestingModule, {
            beforeDestroy: spy,
        });
        const vm = new Vue(reactiveOptions);
        vm.$destroy();
        yield vm.$nextTick();
        expect(spy.calledOnce).toBeTruthy();
    }));
});
